@@ -38,6 +38,8 @@
 
 #define QT_FEATURE_c11 1
 
+#define QT_FEATURE_reduce_relocations -1
+
 #define QT_FEATURE_signaling_nan 1
 
 #define QT_FEATURE_zstd 1
@@ -51,14 +53,13 @@
 #define QT_FEATURE_dbus 1
 
 
-#define QT_VERSION_STR "6.3.2"
+#define QT_VERSION_STR "6.4.1"
 
 #define QT_VERSION_MAJOR 6
 
-#define QT_VERSION_MINOR 3
+#define QT_VERSION_MINOR 4
 
-#define QT_VERSION_PATCH 2
+#define QT_VERSION_PATCH 1
 #define QT_LARGEFILE_SUPPORT 64
 #define QT_VISIBILITY_AVAILABLE
-#define QT_COMPILER_SUPPORTS_SIMD_ALWAYS 1
 #define QT_COMPILER_SUPPORTS_NEON 1
