@@ -1,38 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
-**
-** This file is part of the Qt Gui module
-**
-** $QT_BEGIN_LICENSE:LGPL3$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QRHID3D11_P_H
 #define QRHID3D11_P_H
@@ -54,7 +21,7 @@
 #include <QWindow>
 
 #include <d3d11_1.h>
-#include <dxgi1_3.h>
+#include <dxgi1_6.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -93,13 +60,14 @@ struct QD3D11RenderBuffer : public QRhiRenderBuffer
     ID3D11RenderTargetView *rtv = nullptr;
     DXGI_FORMAT dxgiFormat;
     DXGI_SAMPLE_DESC sampleDesc;
+    uint generation = 0;
     friend class QRhiD3D11;
 };
 
 struct QD3D11Texture : public QRhiTexture
 {
-    QD3D11Texture(QRhiImplementation *rhi, Format format, const QSize &pixelSize,
-                  int sampleCount, Flags flags);
+    QD3D11Texture(QRhiImplementation *rhi, Format format, const QSize &pixelSize, int depth,
+                  int arraySize, int sampleCount, Flags flags);
     ~QD3D11Texture();
     void destroy() override;
     bool create() override;
@@ -109,14 +77,21 @@ struct QD3D11Texture : public QRhiTexture
     bool prepareCreate(QSize *adjustedSize = nullptr);
     bool finishCreate();
     ID3D11UnorderedAccessView *unorderedAccessViewForLevel(int level);
+    ID3D11Resource *textureResource() const
+    {
+        if (tex)
+            return tex;
+        return tex3D;
+    }
 
     ID3D11Texture2D *tex = nullptr;
+    ID3D11Texture3D *tex3D = nullptr;
     bool owns = true;
     ID3D11ShaderResourceView *srv = nullptr;
     DXGI_FORMAT dxgiFormat;
     uint mipLevelCount = 0;
     DXGI_SAMPLE_DESC sampleDesc;
-    ID3D11UnorderedAccessView *perLevelViews[QRhi::MAX_LEVELS];
+    ID3D11UnorderedAccessView *perLevelViews[QRhi::MAX_MIP_LEVELS];
     uint generation = 0;
     friend class QRhiD3D11;
 };
@@ -140,6 +115,8 @@ struct QD3D11RenderPassDescriptor : public QRhiRenderPassDescriptor
     ~QD3D11RenderPassDescriptor();
     void destroy() override;
     bool isCompatible(const QRhiRenderPassDescriptor *other) const override;
+    QRhiRenderPassDescriptor *newCompatibleRenderPassDescriptor() const override;
+    QVector<quint32> serializedFormat() const override;
 };
 
 struct QD3D11RenderTargetData
@@ -160,12 +137,14 @@ struct QD3D11RenderTargetData
     static const int MAX_COLOR_ATTACHMENTS = 8;
     ID3D11RenderTargetView *rtv[MAX_COLOR_ATTACHMENTS];
     ID3D11DepthStencilView *dsv = nullptr;
+
+    QRhiRenderTargetAttachmentTracker::ResIdList currentResIdList;
 };
 
-struct QD3D11ReferenceRenderTarget : public QRhiRenderTarget
+struct QD3D11SwapChainRenderTarget : public QRhiSwapChainRenderTarget
 {
-    QD3D11ReferenceRenderTarget(QRhiImplementation *rhi);
-    ~QD3D11ReferenceRenderTarget();
+    QD3D11SwapChainRenderTarget(QRhiImplementation *rhi, QRhiSwapChain *swapchain);
+    ~QD3D11SwapChainRenderTarget();
     void destroy() override;
 
     QSize pixelSize() const override;
@@ -202,6 +181,7 @@ struct QD3D11ShaderResourceBindings : public QRhiShaderResourceBindings
     ~QD3D11ShaderResourceBindings();
     void destroy() override;
     bool create() override;
+    void updateResources(UpdateFlags flags) override;
 
     bool hasDynamicOffset = false;
     QVarLengthArray<QRhiShaderResourceBinding, 8> sortedBindings;
@@ -277,7 +257,7 @@ struct QD3D11ShaderResourceBindings : public QRhiShaderResourceBindings
     friend class QRhiD3D11;
 };
 
-Q_DECLARE_TYPEINFO(QD3D11ShaderResourceBindings::BoundResourceData, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QD3D11ShaderResourceBindings::BoundResourceData, Q_RELOCATABLE_TYPE);
 
 struct QD3D11GraphicsPipeline : public QRhiGraphicsPipeline
 {
@@ -435,6 +415,7 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
                 UINT dstSubRes;
                 UINT dstX;
                 UINT dstY;
+                UINT dstZ;
                 ID3D11Resource *src;
                 UINT srcSubRes;
                 bool hasSrcBox;
@@ -518,9 +499,6 @@ struct QD3D11CommandBuffer : public QRhiCommandBuffer
         currentGraphicsPipeline = nullptr;
         currentComputePipeline = nullptr;
         currentPipelineGeneration = 0;
-        resetCachedShaderResourceState();
-    }
-    void resetCachedShaderResourceState() {
         currentGraphicsSrb = nullptr;
         currentComputeSrb = nullptr;
         currentSrbGeneration = 0;
@@ -542,6 +520,8 @@ struct QD3D11SwapChain : public QRhiSwapChain
     QRhiRenderTarget *currentFrameRenderTarget() override;
 
     QSize surfacePixelSize() override;
+    bool isFormatSupported(Format f) override;
+    QRhiSwapChainHdrInfo hdrInfo() override;
 
     QRhiRenderPassDescriptor *newCompatibleRenderPassDescriptor() override;
     bool createOrResize() override;
@@ -552,10 +532,12 @@ struct QD3D11SwapChain : public QRhiSwapChain
 
     QWindow *window = nullptr;
     QSize pixelSize;
-    QD3D11ReferenceRenderTarget rt;
+    QD3D11SwapChainRenderTarget rt;
     QD3D11CommandBuffer cb;
     DXGI_FORMAT colorFormat;
+    DXGI_FORMAT srgbAdjustedColorFormat;
     IDXGISwapChain *swapChain = nullptr;
+    UINT swapChainFlags = 0;
     static const int BUFFER_COUNT = 2;
     ID3D11Texture2D *backBufferTex;
     ID3D11RenderTargetView *backBufferRtv;
@@ -592,6 +574,8 @@ public:
                                          QRhiTexture::Format backingFormatHint) override;
     QRhiTexture *createTexture(QRhiTexture::Format format,
                                const QSize &pixelSize,
+                               int depth,
+                               int arraySize,
                                int sampleCount,
                                QRhiTexture::Flags flags) override;
     QRhiSampler *createSampler(QRhiSampler::Filter magFilter,
@@ -671,11 +655,15 @@ public:
     bool isFeatureSupported(QRhi::Feature feature) const override;
     int resourceLimit(QRhi::ResourceLimit limit) const override;
     const QRhiNativeHandles *nativeHandles() override;
-    void sendVMemStatsToProfiler() override;
+    QRhiDriverInfo driverInfo() const override;
+    QRhiMemAllocStats graphicsMemoryAllocationStatistics() override;
     bool makeThreadLocalNativeContextCurrent() override;
     void releaseCachedResources() override;
     bool isDeviceLost() const override;
 
+    QByteArray pipelineCacheData() override;
+    void setPipelineCacheData(const QByteArray &data) override;
+
     void enqueueSubresUpload(QD3D11Texture *texD, QD3D11CommandBuffer *cbD,
                              int layer, int level, const QRhiTextureSubresourceUploadDescription &subresDesc);
     void enqueueResourceUpdates(QRhiCommandBuffer *cb, QRhiResourceUpdateBatch *resourceUpdates);
@@ -691,7 +679,10 @@ public:
     void finishActiveReadbacks();
     void reportLiveObjects(ID3D11Device *device);
     void clearShaderCache();
+    QByteArray compileHlslShaderSource(const QShader &shader, QShader::Variant shaderVariant, uint flags,
+                                       QString *error, QShaderKey *usedShaderKey);
 
+    QRhi::Flags rhiFlags;
     bool debugLayer = false;
     bool importedDeviceAndContext = false;
     ID3D11Device *dev = nullptr;
@@ -699,11 +690,14 @@ public:
     D3D_FEATURE_LEVEL featureLevel = D3D_FEATURE_LEVEL(0);
     LUID adapterLuid = {};
     ID3DUserDefinedAnnotation *annotations = nullptr;
+    IDXGIAdapter1 *activeAdapter = nullptr;
     IDXGIFactory1 *dxgiFactory = nullptr;
-    bool hasDxgi2 = false;
-    bool supportsFlipDiscardSwapchain = false;
+    bool supportsFlipSwapchain = false;
+    bool supportsAllowTearing = false;
+    bool forceFlipDiscard = false;
     bool deviceLost = false;
     QRhiD3D11NativeHandles nativeHandlesStruct;
+    QRhiDriverInfo driverInfoStruct;
 
     struct {
         int vsHighestActiveVertexBufferBinding = -1;
@@ -760,10 +754,44 @@ public:
         void releaseResources();
         void activate();
     } deviceCurse;
+
+    // This is what gets exposed as the "pipeline cache", not that that concept
+    // applies anyway. Here we are just storing the DX bytecode for a shader so
+    // we can skip the HLSL->DXBC compilation when the QShader has HLSL source
+    // code and the same shader source has already been compiled before.
+    // m_shaderCache seemingly does the same, but this here does not care about
+    // the ID3D11*Shader, this is just about the bytecode and about allowing
+    // the data to be serialized to persistent storage and then reloaded in
+    // future runs of the app, or when creating another QRhi, etc.
+    struct BytecodeCacheKey {
+        QByteArray sourceHash;
+        QByteArray target;
+        QByteArray entryPoint;
+        uint compileFlags;
+    };
+    QHash<BytecodeCacheKey, QByteArray> m_bytecodeCache;
 };
 
-Q_DECLARE_TYPEINFO(QRhiD3D11::TextureReadback, Q_MOVABLE_TYPE);
-Q_DECLARE_TYPEINFO(QRhiD3D11::BufferReadback, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QRhiD3D11::TextureReadback, Q_RELOCATABLE_TYPE);
+Q_DECLARE_TYPEINFO(QRhiD3D11::BufferReadback, Q_RELOCATABLE_TYPE);
+
+inline bool operator==(const QRhiD3D11::BytecodeCacheKey &a, const QRhiD3D11::BytecodeCacheKey &b) noexcept
+{
+    return a.sourceHash == b.sourceHash
+            && a.target == b.target
+            && a.entryPoint == b.entryPoint
+            && a.compileFlags == b.compileFlags;
+}
+
+inline bool operator!=(const QRhiD3D11::BytecodeCacheKey &a, const QRhiD3D11::BytecodeCacheKey &b) noexcept
+{
+    return !(a == b);
+}
+
+inline size_t qHash(const QRhiD3D11::BytecodeCacheKey &k, size_t seed = 0) noexcept
+{
+    return qHash(k.sourceHash, seed) ^ qHash(k.target) ^ qHash(k.entryPoint) ^ k.compileFlags;
+}
 
 QT_END_NAMESPACE
 
